package fr.inra.pappso.FastaManager.database;

import fr.inra.pappso.FastaManager.FastaFile.FastaFile;
import fr.inra.pappso.FastaManager.FastaFile.FastaFileFactory;
import fr.inra.pappso.FastaManager.exception.FastaException;
import fr.inra.pappso.FastaManager.fasta_transforme.output.FastaOutputFile;
import fr.inra.pappso.FastaManager.fasta_transforme.output.FastaWriter;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.Enumeration;

import org.apache.log4j.Logger;
import org.w3c.dom.*;

import javax.xml.parsers.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

public class Repository {
	private static final Logger logger = Logger.getLogger(Repository.class);

	protected Hashtable<String, Element> databases;

	protected String file = "database_manager.xml";

	protected String database_repository;

	protected Document doc;

	protected Element racine;

	protected Element repositorie;

	protected boolean readOnly = true;

	public Repository(Element repositorie) throws Exception {
		// Creation fichier xml
		this.repositorie = repositorie;
		this.databases = new Hashtable<String, Element>();
		this.file = repositorie.getAttribute("resource") + "/" + this.file;

		// creation et ouverture d'un DOM
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder parser = factory.newDocumentBuilder();

		if (!this.isXmlExist()) {
			this.doc = parser.newDocument();
			this.racine = doc.createElement("database_manager");
			doc.appendChild(this.racine);
			Element repo = doc.createElement("repository");
			repo.setAttribute("value", repositorie.getAttribute("resource"));
			this.racine.appendChild(repo);
			write();
		} else {
			this.doc = parser.parse(this.getXmlInputStream());
			this.racine = this.doc.getDocumentElement();
		}
		readOnly = this.islock();
		if (!readOnly) {
			this.lock();
		}
		initialise();
	}

	protected InputStream getXmlInputStream() throws Exception {
		return (new FileInputStream(file));
	}

	protected void writeToFile(InputStream is) throws Exception {
		FileOutputStream output = new FileOutputStream(file);

		int n = 8000;

		byte b[] = new byte[n];
		int nb_bytes;
		while ((nb_bytes = is.read(b)) > 0) {
			output.write(b, 0, nb_bytes);
		}
		is.close();
		output.flush();
		output.close();
	}

	protected boolean isXmlExist() throws Exception {
		File f = new File(file);
		return (f.exists());
	}

	protected void initialise() {
		// Remplissage des index id de database
		NodeList nl = this.racine.getElementsByTagName("database");
		databases.clear();
		for (int i = 0; i < nl.getLength(); ++i) {
			Element database = (Element) nl.item(i);
			String id = database.getAttribute("id");
			databases.put(id, database);
		}
		// remplissage de l'element repositorie
		NodeList nl2 = this.racine.getElementsByTagName("repository");
		Element repositorie = (Element) nl2.item(0);
		this.database_repository = repositorie.getAttribute("value");
	}

	public Element getRepositorie() {
		return repositorie;
	}

	public Hashtable<String, Element> get_databases() {
		return (this.databases);
	}

	public String get_database_repository() {
		return (this.database_repository);
	}

	public String get_source_database(Element element) {
		// Element database = (Element) o;
		String species = element.getAttribute("source_database");
		return (species);
	}

	public String get_species(Element element) {
		// Element database = (Element) o;
		String species = element.getAttribute("species");
		return (species);
	}

	public String get_version(Element element) {
		// Element database = (Element) o;
		String species = element.getAttribute("version");
		return (species);
	}

	public String get_type(Element element) {
		// Element database = (Element) o;
		String species = element.getAttribute("type");
		return (species);
	}

	public FastaFile getFastaFile(Element element) {

		String species = element.getAttribute("fasta_file");
		FastaFile fastaFile = FastaFileFactory.newFastaFile(this
				.get_database_repository() + "/" + species);
		// Element database = (Element) o;
		return (fastaFile);
	}

	public String get_entries(Element element) {
		// Element database = (Element) o;
		String species = element.getAttribute("entries");
		return (species);
	}

	public String get_site_web(Element element) {
		// Element database = (Element) o;
		String species = element.getAttribute("site_web");
		return (species);
	}

	public String get_description(Element element) {
		// Element database = (Element) o;
		String species = element.getTextContent();
		return (species);
	}

	public String get_id(Element element) {
		// Element database = (Element) o;
		String att = element.getAttribute("id");
		return (att);
	}

	public String get_sequence_type(Element element) {
		// Element database = (Element) o;
		String att = element.getAttribute("sequence_type");
		return (att);
	}

	public String get_combinaison(Element element) {
		// Element database = (Element) o;
		String att = element.getAttribute("combine");
		return (att);
	}

	public String get_database_information(Object o) {
		Element database = (Element) o;
		String info = "Id :\t" + database.getAttribute("id") + "\n";
		info = info + "Source :\t" + database.getAttribute("source_database")
				+ "\n";
		info = info + "Species :\t" + database.getAttribute("species") + "\n";
		info = info + "Version :\t" + database.getAttribute("version") + "\n";
		info = info + "Entries :\t" + database.getAttribute("entries") + "\n";
		info = info + "Type :\t" + database.getAttribute("type") + "\n";
		info = info + "Sequence Type :\t"
				+ database.getAttribute("sequence_type") + "\n";
		info = info + "Web site :\t" + database.getAttribute("site_web") + "\n";
		info = info + "Description :\n" + database.getTextContent() + "\n\n";
		return (info);
	}

	public void set_fasta_file(Object o, String S) {
		Element database = (Element) o;
		database.setAttribute("fasta_file", S);
	}

	public void set_sequence_type(Object o, String S) {
		Element database = (Element) o;
		database.setAttribute("sequence_type", S);
	}

	public void renameFastaFile(Object o, String S) throws Exception {
		Element database = (Element) o;
		try {
			FastaFile fastaFile = this.getFastaFile(database);
			if (fastaFile.renameTo(S)) {
				database.setAttribute("fasta_file", S);
			} else {
				String message = "rename fasta file failed";
				logger.error(message);
				throw new FastaException(message);
			}
		} catch (Exception e) {
			throw (e);
		}
	}

	public void set_combinaison(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("combine", Value);
	}

	public void set_source_database(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("source_database", Value);
	}

	public void set_species(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("species", Value);
	}

	public void set_version(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("version", Value);
	}

	public void set_type(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("type", Value);
	}

	public void set_entries(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("entries", Value);
	}

	public void set_site_web(Object o, String Value) {
		Element database = (Element) o;
		database.setAttribute("site_web", Value);
	}

	public void set_description(Object o, String S) {
		Element database = (Element) o;
		database.setTextContent(S);
	}

	public void add_database() {
		String id = select_new_id();
		Element database = doc.createElement("database");
		database.setAttribute("combine", "");
		database.setAttribute("id", id);
		database.setAttribute("source_database", "");
		database.setAttribute("species", "");
		database.setAttribute("version", "");
		database.setAttribute("type", "");
		database.setAttribute("sequence_type", "");
		database.setAttribute("fasta_file", "");
		database.setAttribute("entries", "");
		database.setAttribute("site_web", "");
		database.setTextContent("");
		this.racine.appendChild(database);
		initialise();
	}

	public void write() throws Exception {
		// écriture d'un document
		try {
			TransformerFactory tfact = TransformerFactory.newInstance();
			Transformer transformer = tfact.newTransformer();
			transformer.setOutputProperty("encoding", "UTF-8");
			DOMSource source = new DOMSource(this.doc);
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			StreamResult result = new StreamResult(output);
			transformer.transform(source, result);
			this.writeToFile(new ByteArrayInputStream(output.toByteArray()));

		} catch (Exception e) {
			throw new FastaException("Problems during saved of the file : "
					+ this.file);
		}
	}

	public void deleteFastaFile(Element repos) throws Exception {
		FastaFile fastaFile = this.getFastaFile(repos);
		// on enlève l'element
		racine.removeChild(repos);
		initialise();
		// on essaie d'enlever la base donnée si elle existe
		try {
			if (fastaFile.exists()) {
				if (fastaFile.delete() == false) {
					String message = "unable to delete file "
							+ fastaFile.getName();
					logger.error(message);
					throw new FastaException(message);
				}
			}
		} catch (Exception e) {
			throw (e);
		}
	}

	public String select_new_id() {
		String id = "db";
		boolean id_ok = false;
		int i = 0;
		do {
			i++;
			id_ok = true;
			Enumeration<String> E = this.databases.keys();
			while (E.hasMoreElements()) {
				String cle = E.nextElement();
				Element database = (Element) this.databases.get(cle);
				if (database.getAttribute("id").equals(id + i))
					id_ok = false;
			}

		} while (!id_ok);
		id = id + i;
		return (id);
	}

	public FastaWriter newFastaWriter() throws IOException, FastaException {
		String filename = get_database_repository()
				+ System.getProperty("file.separator") + this.select_new_id()
				+ ".fasta";
		logger.debug(filename);
		return new FastaOutputFile(filename);
	}

	public void unlock() throws Exception {
		File locking = new File(this.file + ".lock");
		if (!this.readOnly && locking.exists()) {
			locking.delete();
		}

	}

	public void lock() throws Exception {
		File locking = new File(this.file + ".lock");
		locking.createNewFile();
	}

	public boolean islock() throws Exception {
		File locking = new File(this.file + ".lock");
		return (locking.exists());
	}

	public boolean isReadOnly() {
		return (readOnly);
	}

}
