package fr.inra.pappso.FastaManager.gui;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import org.apache.log4j.Logger;
import org.w3c.dom.Element;

import fr.inra.pappso.FastaManager.database_manager;
import fr.inra.pappso.FastaManager.FastaFile.FastaFile;
import fr.inra.pappso.FastaManager.FastaFile.FastaFileFactory;
import fr.inra.pappso.FastaManager.database.Repository;
import fr.inra.pappso.FastaManager.database.RepositorysLocation;
import fr.inra.pappso.FastaManager.exception.FastaException;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

public class main_window extends JFrame implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static final Logger logger = Logger.getLogger(main_window.class);

	private JTable table;

	private DefaultTableModel tableModel;

	private Repository repository;

	private RepositorysLocation repositories;

	private Hashtable<String, Element> ids;

	private JMenu repositoryMenu;

	private JLabel information;

	private Vector<JMenuItem> menuItemLock;

	public main_window() {
		super("Database Manager v2.0");
		try {
			// inititalisation du xml et de la table
			this.repositories = new RepositorysLocation();
			this.repository = null;
			this.menuItemLock = new Vector<JMenuItem>();
			tableModel = new DefaultTableModel();
			table = new JTable(tableModel);
			table.setAutoCreateRowSorter(true);

			// Creation de la fenetre principale (boutons...)
			JFrame_initialise();

			// mettre la fenetre visible
			setVisible(true);

		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	private void JFrame_initialise() throws Exception {
		// Creation des menus
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		menuBar.add(fileMenu);
		repositoryMenu = new JMenu("Repositories");
		menuBar.add(repositoryMenu);
		JMenu fastaMenu = new JMenu("Fasta");
		menuBar.add(fastaMenu);
		JMenu about = new JMenu("?");
		menuBar.add(about);

		// creation des sous-menu
		JMenuItem addLocal = new JMenuItem("Add local repository");
		JMenuItem addWebdav = new JMenuItem("Add webdav repository");
		JMenuItem close = new JMenuItem("Close");
		JMenuItem importMenu = new JMenuItem("Import");
		JMenuItem exportMenu = new JMenuItem("Export");
		JMenuItem editMenu = new JMenuItem("Edit");
		JMenuItem transformMenu = new JMenuItem("Transforme");
		JMenuItem combineMenu = new JMenuItem("Combine");
		JMenuItem deleteMenu = new JMenuItem("Delete");
		JMenuItem aboutMenu = new JMenuItem("About");
		JMenuItem newsMenu = new JMenuItem("News");

		// ajout de menus à bloquer
		this.menuItemLock.add(importMenu);
		this.menuItemLock.add(editMenu);
		this.menuItemLock.add(transformMenu);
		this.menuItemLock.add(combineMenu);
		this.menuItemLock.add(deleteMenu);
		this.modifyLockMenuItem(false);

		// Ajout des sous-menu au menu
		fileMenu.add(addLocal);
		fileMenu.add(addWebdav);
		fileMenu.add(close);
		fastaMenu.add(importMenu);
		fastaMenu.add(exportMenu);
		fastaMenu.add(editMenu);
		fastaMenu.add(transformMenu);
		fastaMenu.add(combineMenu);
		fastaMenu.add(deleteMenu);
		about.add(newsMenu);
		about.add(aboutMenu);

		// creation des actions sur les menus
		addLocal.addActionListener(this);
		addWebdav.addActionListener(this);
		close.addActionListener(this);
		exportMenu.addActionListener(this);
		importMenu.addActionListener(this);
		editMenu.addActionListener(this);
		transformMenu.addActionListener(this);
		combineMenu.addActionListener(this);
		deleteMenu.addActionListener(this);
		aboutMenu.addActionListener(this);
		newsMenu.addActionListener(this);

		// creation des boutons //information
		information = new JLabel("Open or create repository...");
		// JButton modify = new JButton("Modify");
		JButton removeRepo = new JButton("Remove");

		// creation des actions sur les boutons
		// modify.addActionListener(this);
		removeRepo.addActionListener(this);

		// ajout des boutons dans le panel
		JPanel panel = new JPanel();
		panel.add(information);
		// panel.add(modify);
		panel.add(removeRepo);

		// initialisation de la fenetre
		this.setJMenuBar(menuBar);
		this.updateRepositoriesMenu();
		this.add(panel, BorderLayout.SOUTH);
		this.add(new JScrollPane(table), BorderLayout.CENTER);
		this.setSize(1000, 480);

		// initialisation de l'operation de fermeture de la fenetre
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				main_window.this.dispose();
			}
		});

		// centrage de la fenetre
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(dim.width / 2 - getWidth() / 2, dim.height / 2
				- getHeight() / 2);

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			UIManager.setLookAndFeel(UIManager
					.getCrossPlatformLookAndFeelClassName());
			MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
			UIManager.setLookAndFeel(new MetalLookAndFeel());
		}
	}

	public void dispose() {
		logger.info("Close application");
		if (repository != null) {
			try{
			repository.unlock();
			}catch (Exception e){
				logger.error(e);
			}
			logger.info("Unlock repository");
		}
		super.dispose();
		System.exit(0);
	}

	private void updateRepositoriesMenu() {
		this.repositoryMenu.removeAll();
		Enumeration<Element> repos = repositories.getRepositories().elements();
		while (repos.hasMoreElements()) {
			Element repo = repos.nextElement();
			JMenuItem repoMenu = new JMenuItem(
					repositories.getRepositorySignet(repo));
			repoMenu.addActionListener(this);
			this.repositoryMenu.add(repoMenu);
		}
	}

	private void modifyLockMenuItem(boolean enabled) {
		for (JMenuItem menuItem : this.menuItemLock) {
			menuItem.setEnabled(enabled);
		}

	}

	private void JTable_initialise() {
		if (repository != null) {
			this.ids = repository.get_databases();
		} else {
			this.ids = new Hashtable<String, Element>();
		}
		Vector<Vector<String>> rowData = new Vector<Vector<String>>();
		// Vector row = new Vector();
		Enumeration<String> E = this.ids.keys();
		while (E.hasMoreElements()) {
			String cle = E.nextElement();
			Vector<String> row = new Vector<String>();
			row.add(repository.get_id(ids.get(cle)));
			row.add(repository.get_source_database(ids.get(cle)));
			row.add(repository.get_species(ids.get(cle)));
			row.add(repository.get_version(ids.get(cle)));
			row.add(repository.get_type(ids.get(cle)));
			row.add(repository.get_entries(ids.get(cle)));
			row.add(repository.getFastaFile(ids.get(cle)).getName());
			rowData.add(row);
		}

		Vector<String> columnNames = new Vector<String>();
		columnNames.addElement("Id");
		columnNames.addElement("Source");
		columnNames.addElement("Species");
		columnNames.addElement("Version");
		columnNames.addElement("Type");
		columnNames.addElement("Entries");
		columnNames.addElement("Files");
		tableModel.setDataVector(rowData, columnNames);

		// centrer certaine cellules
		DefaultTableCellRenderer custom = new DefaultTableCellRenderer();
		custom.setHorizontalAlignment(JLabel.CENTER);
		table.getColumnModel().getColumn(0).setCellRenderer(custom);
		table.getColumnModel().getColumn(1).setCellRenderer(custom);
		table.getColumnModel().getColumn(3).setCellRenderer(custom);
		table.getColumnModel().getColumn(5).setCellRenderer(custom);

		table.repaint();
	}

	private void addNewLocalRepository() {
		try {
			if (this.repository != null) {
				this.repository.unlock();
			}
			repository_window win = new repository_window(this, true,
					this.repositories, RepositorysLocation.LocalRepository);
			if (win.getRepository() == null)
				throw new FastaException("");
			this.repository = win.getRepository();
			this.information.setText(this.repositories
					.getInformation(repository.getRepositorie()));
			this.updateRepositoriesMenu();
			this.modifyLockMenuItem(true);
			JTable_initialise();

		} catch (Exception e) {
			this.showError(e, "Problems during add New Local Repository");
		}
	}

	private void showError(Exception e, String mess) {
		logger.error(e);
		JOptionPane.showMessageDialog(this, mess + "\n" + e.getMessage(),
				"Error", JOptionPane.ERROR_MESSAGE);
	}

	private void addNewWebdavRepository() {
		try {
			if (this.repository != null) {
				this.repository.unlock();
			}
			repository_window win = new repository_window(this, true,
					this.repositories, RepositorysLocation.WebdavRepository);
			if (win.getRepository() == null)
				throw new FastaException("");
			this.repository = win.getRepository();
			this.information.setText(this.repositories
					.getInformation(repository.getRepositorie()));
			this.updateRepositoriesMenu();
			this.modifyLockMenuItem(true);
			this.JTable_initialise();

		} catch (Exception e) {
			this.showError(e, "Problems during add New Webdav Repository");
		}
	}

	private void imports_clic(ActionEvent evt) {
		try {
			// sélection d'un fichier fasta
			File selectedFile;
			JFileChooser fileChooser = new JFileChooser(
					repository.get_database_repository());
			fileChooser.setDialogTitle("Select the fasta file to import");
			fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			fileChooser.setFileSelectionMode(JFileChooser.OPEN_DIALOG);
			fileChooser.setMultiSelectionEnabled(false);
			// fileChooser.setCurrentDirectory(new File
			// (xml.get_database_repository()));
			int returnValue = fileChooser.showOpenDialog(null);
			if (returnValue == JFileChooser.CANCEL_OPTION)
				throw new FastaException("No fasta file select");
			selectedFile = fileChooser.getSelectedFile();
			ArrayList<FastaFile> files = new ArrayList<FastaFile>(0);
			files.add(FastaFileFactory.newFastaFile(selectedFile));

			// selection d'un nouvel id pour vérifier l'importation
			String id = repository.select_new_id();

			// Fenetre d'importation
			new transfert_window(this, true, repository, "Inconnu", id, files);
			// on continue si il y a eu une base de donnée ajoutée!
			this.ids = repository.get_databases();
			if (ids.containsKey(id)) {
				repository.write();
				JOptionPane.showMessageDialog(this,
						"Add of the database finish : " + id, "Message",
						JOptionPane.INFORMATION_MESSAGE);

				// Suppression de la base de donnée
				// if (selectedFile.exists()) {
				// selectedFile.delete();
				// }
				new edit_window(this, true, repository, id);

			}
		} catch (Exception e) {
			this.showError(e, "Problems during database import");
		} finally {
			JTable_initialise();
		}
	}

	private void edit_clic(ActionEvent evt) {
		try {
			int i = table.getSelectedRowCount();
			if (i == 0) {
				throw new FastaException(
						"This Action is possible with only one database selected");
			} else if (i > 1) {
				throw new FastaException(
						"This Action is not possible with multiple database selected");

			} else {
				String id = (String) table
						.getValueAt(table.getSelectedRow(), 0);
				if (!ids.containsKey(id))
					throw new FastaException("This id : " + id
							+ " doesn't exist");
				new edit_window(this, true, repository, id);
			}
		} catch (Exception e) {
			this.showError(e, "Problems during edition");
		} finally {
			JTable_initialise();
		}

	}

	private void export_clic(ActionEvent evt) {
		try {
			int[] i = table.getSelectedRows();
			if (i.length == 0)
				throw new FastaException(
						"This Action is possible with only one database selected");
			String selection = "";
			String id_value = "";
			for (int j = 0; j < i.length; j++) {
				String id = (String) table.getValueAt(i[j], 0);
				if (!ids.containsKey(id))
					throw new FastaException("This id : " + id
							+ " doesn't exist");
				id_value = id_value + id + " ";
				selection = selection
						+ repository.get_database_information(ids.get(id));
			}
			StringSelection data = new StringSelection(selection);
			Clipboard clipboard = getToolkit().getSystemClipboard();
			clipboard.setContents(data, data);
			JOptionPane.showMessageDialog(this, "The database metadatas : "
					+ id_value + "have been copy on the clipboard", "Message",
					JOptionPane.INFORMATION_MESSAGE);
		} catch (Exception e) {
			this.showError(e, "Problems during export");
		}
	}

	private void transforme_clic(ActionEvent evt) {
		try {
			int[] i = table.getSelectedRows();
			if (i.length != 1)
				throw new FastaException(
						"This Action is possible with only one database selected");

			// récupération de l'ensemble des ids et remplissage des fichiers
			String id_sel = (String) table.getValueAt(i[0], 0);
			String type = repository.get_sequence_type(ids.get((String) table
					.getValueAt(i[0], 0)));
			ArrayList<FastaFile> files = new ArrayList<FastaFile>();
			files.add(repository.getFastaFile(ids.get(id_sel)));
			// selection d'un nouvel id pour vérifier l'importation
			String id = repository.select_new_id();

			// Fenetre d'importation
			new transfert_window(this, true, repository, type, id, files);
			// on continue si il y a eu une base de donnée ajoutée!
			this.ids = repository.get_databases();
			if (ids.containsKey(id)) {
				repository.set_combinaison(ids.get(id),
						repository.get_id(ids.get(id_sel)));
				repository.set_source_database(ids.get(id),
						repository.get_source_database(ids.get(id_sel)));
				repository.set_species(ids.get(id),
						repository.get_species(ids.get(id_sel)));
				repository.set_version(ids.get(id),
						repository.get_version(ids.get(id_sel)));
				repository.set_site_web(ids.get(id),
						repository.get_site_web(ids.get(id_sel)));
				repository.set_description(ids.get(id), "Transformation of :\n"
						+ repository.get_description(ids.get(id_sel)));

				repository.write();
				JOptionPane.showMessageDialog(this,
						"Transformation of the database finish : " + id,
						"Message", JOptionPane.INFORMATION_MESSAGE);
			}
		} catch (Exception e) {
			this.showError(e, "Problems during database transformation");
		} finally {
			JTable_initialise();
		}
	}

	private void combine_clic(ActionEvent evt) {
		try {
			int[] i = table.getSelectedRows();
			if (i.length <= 1)
				throw new FastaException(
						"This Action is possible with at less 2 database selected");

			// récupération de l'ensemble des ids et remplissage des fichiers
			String type = repository.get_sequence_type(ids.get((String) table
					.getValueAt(i[0], 0)));
			String[] id_sel = new String[i.length];
			ArrayList<FastaFile> files = new ArrayList<FastaFile>();
			for (int j = 0; j < i.length; j++) {
				id_sel[j] = (String) table.getValueAt(i[j], 0);
				if (!type.equals(repository.get_sequence_type(ids
						.get(id_sel[j]))))
					throw new FastaException(
							"This Action is possible with only database of similar sequence type (Prot ou Est)");

				files.add(repository.getFastaFile(ids.get(id_sel[j])));
			}

			// selection d'un nouvel id pour vérifier l'importation
			String id = repository.select_new_id();

			// Fenetre d'importation
			new transfert_window(this, true, repository, type, id, files);
			// on continue si il y a eu une base de donnée ajoutée!
			this.ids = repository.get_databases();
			if (ids.containsKey(id)) {
				repository.set_combinaison(ids.get(id),
						repository.get_id(ids.get(id_sel[0])));
				repository.set_source_database(ids.get(id),
						repository.get_source_database(ids.get(id_sel[0])));
				repository.set_species(ids.get(id),
						repository.get_species(ids.get(id_sel[0])));
				repository.set_version(ids.get(id),
						repository.get_version(ids.get(id_sel[0])));
				repository.set_type(ids.get(id),
						repository.get_type(ids.get(id_sel[0])));
				repository.set_site_web(ids.get(id),
						repository.get_site_web(ids.get(id_sel[0])));
				repository.set_description(ids.get(id), "Combined of :\n"
						+ repository.get_description(ids.get(id_sel[0])));

				for (int j = 1; j < id_sel.length; j++) {
					repository.set_combinaison(
							ids.get(id),
							repository.get_id(ids.get(id)) + "|"
									+ repository.get_id(ids.get(id_sel[j])));
					repository.set_source_database(
							ids.get(id),
							repository.get_source_database(ids.get(id))
									+ "|"
									+ repository.get_source_database(ids
											.get(id_sel[j])));
					repository
							.set_species(
									ids.get(id),
									repository.get_species(ids.get(id))
											+ "|"
											+ repository.get_species(ids
													.get(id_sel[j])));
					repository
							.set_version(
									ids.get(id),
									repository.get_version(ids.get(id))
											+ "|"
											+ repository.get_version(ids
													.get(id_sel[j])));
					repository.set_type(
							ids.get(id),
							repository.get_type(ids.get(id)) + "|"
									+ repository.get_type(ids.get(id_sel[j])));
					repository.set_site_web(
							ids.get(id),
							repository.get_site_web(ids.get(id))
									+ "|"
									+ repository.get_site_web(ids
											.get(id_sel[j])));
					repository.set_description(
							ids.get(id),
							repository.get_description(ids.get(id))
									+ "\n\n"
									+ repository.get_description(ids
											.get(id_sel[j])));
				}
				repository.write();
				JOptionPane.showMessageDialog(this,
						"Database combination finish : " + id, "Message",
						JOptionPane.INFORMATION_MESSAGE);
			}
		} catch (Exception e) {
			this.showError(e, "Problems during database combination");
		} finally {
			JTable_initialise();
		}
	}

	private void delete_clic(ActionEvent evt) {
		try {
			int i = table.getSelectedRowCount();
			if (i == 0) {
				throw new FastaException(
						"This Action is possible with only one database selected");
			} else if (i > 1) {
				throw new FastaException(
						"This Action is not possible with multiple databases selected");

			} else {

				String id = (String) table
						.getValueAt(table.getSelectedRow(), 0);
				if (!ids.containsKey(id))
					throw new FastaException("Database id is not found");
				repository.deleteFastaFile(ids.get(id));
				repository.write();
				JOptionPane.showMessageDialog(this,
						"Removed of the database : " + id, "Message",
						JOptionPane.INFORMATION_MESSAGE);
			}
		} catch (Exception e) {
			this.showError(e, "Problems during database removing");
		} finally {
			JTable_initialise();
		}
	}

	private void removeRepository() {
		try {
			if (this.repository != null) {
				this.repository.unlock();
				this.repositories.deleteRepository(repository.getRepositorie());
				this.repositories.write();
				this.information.setText("Open or create repository...");
				this.repository = null;
				this.ids = null;
				this.modifyLockMenuItem(false);
				this.JTable_initialise();
				this.updateRepositoriesMenu();
			}
		} catch (Exception e) {
			this.showError(e, "Problems during delete repository");
		}
	}

	private void loadRepository(String signet) {
		try {
			if (this.repository != null) {
				this.repository.unlock();
			}
			Element repo = this.repositories.getRepositorieElement(signet);
			if (repo != null) {
				this.repository = this.repositories.getRepositorie(repo);
				this.information.setText(this.repositories
						.getInformation(repository.getRepositorie()));
				this.modifyLockMenuItem(!this.repository.isReadOnly());
				this.JTable_initialise();
				if (this.repository.isReadOnly()) {
					JOptionPane.showMessageDialog(this,
							"The current repositoy are still opened\n"
									+ "You have only read access", "Warning",
							JOptionPane.WARNING_MESSAGE);
				}
			}
		} catch (Exception e) {
			this.showError(e, "Problems during loading repository");
		}
	}
	

	private void viewAbout() {
		JOptionPane.showMessageDialog(this,
				database_manager.about, "About",
				JOptionPane.INFORMATION_MESSAGE);
		
	}
	
	private void viewNews() {
		JOptionPane.showMessageDialog(this,
				database_manager.News, "News",
				JOptionPane.INFORMATION_MESSAGE);
		
	}

	public void actionPerformed(ActionEvent evt) {
		String composant = evt.getActionCommand();
		if (composant.equalsIgnoreCase("Import"))
			imports_clic(evt);
		else if (composant.equalsIgnoreCase("Export"))
			export_clic(evt);
		else if (composant.equalsIgnoreCase("Edit"))
			edit_clic(evt);
		else if (composant.equalsIgnoreCase("Transforme"))
			transforme_clic(evt);
		else if (composant.equalsIgnoreCase("Combine"))
			combine_clic(evt);
		else if (composant.equalsIgnoreCase("Delete"))
			delete_clic(evt);
		else if (composant.equalsIgnoreCase("Add local repository"))
			this.addNewLocalRepository();
		else if (composant.equalsIgnoreCase("Add webdav repository"))
			this.addNewWebdavRepository();
		else if (composant.equalsIgnoreCase("Close"))
			this.dispose();
		else if (composant.equalsIgnoreCase("Remove"))
			this.removeRepository();
		else if (composant.equalsIgnoreCase("About"))
			this.viewAbout();
		else if (composant.equalsIgnoreCase("News"))
			this.viewNews();
		else {
			this.loadRepository(composant);
		}
	}

}