package fr.inra.pappso.FastaManager.fasta_transforme;

import fr.inra.pappso.FastaManager.FastaFile.FastaFile;
import fr.inra.pappso.FastaManager.exception.FastaException;
import fr.inra.pappso.FastaManager.fasta_transforme.input.FastaInputStreamParser;
import fr.inra.pappso.FastaManager.fasta_transforme.output.FastaWriter;
import fr.inra.pappso.FastaManager.sequence_transforme.seq_transforme;

import java.io.InputStream;
import java.util.*;

import org.apache.log4j.Logger;

public class FastaTransformer {
	private static final Logger logger = Logger
			.getLogger(FastaTransformer.class);
	// variable
	private FastaWriter seqOut;

	// nom du fichier outfile
	private ArrayList<FastaFile> fasta_infiles;

	// path/nom des fichiers infiles
	// private int[] fasta_infiles_entries;
	private FastaInputStreamTransformer transformer = null;

	// constructeur
	public FastaTransformer(FastaWriter fastaWriter) throws FastaException {
		try {
			seqOut = fastaWriter;
			this.fasta_infiles = new ArrayList<FastaFile>();
		} catch (Exception e) {
			String message = "error in FastaTransformer constructor : \n"
					+ e.getMessage();
			logger.error(message);
			throw new FastaException(message);
		}
	}

	synchronized public int get_fasta_outfile_entries() {
		if (transformer == null) {
			return (0);
		}
		return (transformer.get_fasta_outfile_entries());
	}

	// fonction
	public void transfert_sequences(seq_transforme transforme) throws Exception {
		if (this.seqOut == null)
			throw new FastaException("No fasta outfile defined");
		if (this.fasta_infiles.size() == 0)
			throw new FastaException("No fasta infile defined");
		try {
			transformer = new FastaInputStreamTransformer(seqOut, transforme);
			FastaInputStreamParser fastaParser = new FastaInputStreamParser(
					transformer);

			for (FastaFile seqInFile : this.fasta_infiles) {
				InputStream seqIn = seqInFile.getInputStream();
				fastaParser.parseFastaInputStream(seqIn);
				seqIn.close();
			}
			seqOut.close();
		} catch (Exception e) {
			throw (e);
		}
	}

	public void addFastaFile(FastaFile inputStream) throws FastaException {
		try {
			this.fasta_infiles.add(inputStream);
		} catch (Exception e) {
			String message = "error in FastaTransformer addFastaFile function : \n"
					+ e.getMessage();
			logger.error(message);
			throw new FastaException(message);
		}
	}

}
