package fr.inra.pappso.FastaManager.fasta_transforme.input;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.apache.log4j.Logger;

/**
 * @brief parses a FASTA file
 *
 *        parse a FASTA file, extracts each sequences, look for display id in
 *        database and set sequence if there was no sequence for this display id
 *
 *        test it with wget : wget --post-file=sequences_file.fasta
 *        --user=langella --password=yourpwd
 *        http://host/proticport/sequences/fasta
 *
 * @author langella
 *
 */

public class FastaInputStreamParser {
	private static final Logger logger = Logger
			.getLogger(FastaInputStreamParser.class);
	// private static final Logger logger =
	// Logger.getLogger(FastaInputStreamParser.class);
	private FastaReader reader;

	public FastaInputStreamParser(FastaReader reader) {
		super();
		this.reader = reader;
	}

	protected void newFastaSequence(String header, String strSequence)
			throws Exception {
		this.reader.newFastaSequence(header, strSequence);
	}

	/**
	 * The parsing is done in this method
	 *
	 * @return
	 * @throws Exception
	 */
	public void parseFastaInputStream(InputStream is) throws Exception {
		InputStreamReader isr = new InputStreamReader(is, "Cp850");
		BufferedReader br = new BufferedReader(isr);

		String line = "";
		String header = "";
		StringBuilder sb = new StringBuilder();
		int maxSequenceLength = -1;
		boolean keepGoing = true;
		//line = br.readLine();
		do {
			line = line.trim(); // nice to have but probably not needed
			//logger.debug(line);
			if (line.length() != 0) {
				if (line.startsWith(">")) {
					if (sb.length() > 0) {
						//logger.debug(header + " " + sb.toString());
						newFastaSequence(header, sb.toString());
						if (maxSequenceLength < sb.length()) {
							maxSequenceLength = sb.length();
						}
						sb = new StringBuilder(maxSequenceLength);
					}
					header = line.substring(1);
				} else if (line.startsWith(";")) {
				} else {
					sb.append(line);
				}
			}
			line = br.readLine();
			if (line == null) {
				//logger.debug(header + " " + sb.toString());
				newFastaSequence(header, sb.toString());
				keepGoing = false;
			}
		} while (keepGoing);
		br.close();
		isr.close();
	}
}
